<?php

/**
 * This file is part of the Nette Framework (http://nette.org)
 * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
 * @package Nette\DI
 */



/**
 * Definition used by ContainerBuilder.
 *
 * @author     David Grudl
 * @package Nette\DI
 */
class DIServiceDefinition extends Object
{
	/** @var string  class or interface name */
	public $class;

	/** @var DIStatement */
	public $factory;

	/** @var DIStatement[] */
	public $setup = array();

	/** @var array */
	public $parameters = array();

	/** @var array */
	public $tags = array();

	/** @var mixed */
	public $autowired = TRUE;

	/** @var bool */
	public $shared = TRUE;

	/** @var bool */
	public $internal = FALSE;


	public function setClass($class, array $args = array())
	{
		$this->class = $class;
		if ($args) {
			$this->setFactory($class, $args);
		}
		return $this;
	}


	public function setFactory($factory, array $args = array())
	{
		$this->factory = new DIStatement($factory, $args);
		return $this;
	}


	public function setArguments(array $args = array())
	{
		if ($this->factory) {
			$this->factory->arguments = $args;
		} else {
			$this->setClass($this->class, $args);
		}
		return $this;
	}


	public function addSetup($target, $args = NULL)
	{
		$_args=func_get_args(); $this->setup[] = new DIStatement($target, is_array($args) ? $args : array_slice($_args, 1));
		return $this;
	}


	public function setParameters(array $params)
	{
		$this->shared = $this->autowired = FALSE;
		$this->parameters = $params;
		return $this;
	}


	public function addTag($tag, $attrs = TRUE)
	{
		$this->tags[$tag] = $attrs;
		return $this;
	}


	public function setAutowired($on)
	{
		$this->autowired = $on;
		return $this;
	}


	public function setShared($on)
	{
		$this->shared = (bool) $on;
		$this->autowired = $this->shared ? $this->autowired : FALSE;
		return $this;
	}


	public function setInternal($on)
	{
		$this->internal = (bool) $on;
		return $this;
	}

}
